Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import('chrome://acewebextension-modules/content/prefmanager.js');
Components.utils.import('chrome://acewebextension-modules/content/util.js');
Components.utils.import('chrome://acewebextension-modules/content/engineStatus.js');
Components.utils.import('chrome://acewebextension-modules/content/news.js');

// this file is the JavaScript backing for the UI wrangling which happens in
// browser.xul. It also initializes the AWE singleton which contains
// all the main injection logic, though that should probably be a proper XPCOM
// service and wouldn't need to be initialized in that case.

function AWE_BrowserUI() {};

AWE_BrowserUI.init = function() {
  window.addEventListener("load", AWE_BrowserUI.chromeLoad, false);
  window.addEventListener("unload", AWE_BrowserUI.chromeUnload, false);
  window.messageManager.addMessageListener('acewebextension:open-in-tab',
      AWE_BrowserUI.openInTab);
};

/**
 * The browser XUL has loaded. Find the elements we need and set up our
 * listeners and wrapper objects.
 */
AWE_BrowserUI.chromeLoad = function(e) {
  var contentAreaContextMenu = document.getElementById("contentAreaContextMenu");
  // Store DOM element references in this object, also for use elsewhere.
  AWE_BrowserUI.tabBrowser = document.getElementById("content");
  AWE_BrowserUI.bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
      .getService(Components.interfaces.nsIStringBundleService)
      .createBundle("chrome://acewebextension/locale/awe-browser.properties");

  // Update visual status when enabled state changes.
  AWE_prefRoot.watch("enabled", AWE_BrowserUI.refreshStatus);
  AWE_BrowserUI.refreshStatus();

  document.getElementById('content').addEventListener(
      'DOMContentLoaded', function(aEvent) {
        var safeWin = aEvent.target.defaultView;
        var href = safeWin.location.href;
        AWE_BrowserUI.checkDisabledScriptNavigation(aEvent, safeWin, href);
        if(windowIsTop(safeWin)) {
            AWE_BrowserUI.checkNewsForUrl(aEvent, safeWin, href);
            AWE_BrowserUI.setVersion(safeWin, AWE_prefRoot.getValue("version"));
            AWE_BrowserUI.setInstalledScripts(safeWin, ["test"]);
        }
      }, true);

  contentAreaContextMenu.addEventListener("popupshowing", AWE_BrowserUI.contextMenuShowing, false);
  contentAreaContextMenu.addEventListener("popuphiding", AWE_ContextMenuCommander.onPopupHiding, false);
    
  AWE_BrowserUI.gmSvc = AWE_util.getService();
  // Reference this once, so that the getter is called at least once, and the
  // initialization routines will run, no matter what.
  AWE_BrowserUI.gmSvc.config;

  // Initialize the chrome side handling of menu commands.
  AWE_MenuCommander.initialize();
  AWE_ContextMenuCommander.initialize();

  AWE_BrowserUI.showToolbarButton();
  
  // start monitoring engine status
  AWE_engineStatus.init();
  
  AWE_news.init();
};

/**
 * Opens the specified URL in a new tab.
 */
AWE_BrowserUI.openTab = function(url) {
  gBrowser.selectedTab = gBrowser.addTab(url);
};

/**
 * Handles tab opening for a GM_openInTab API call.
 */
AWE_BrowserUI.openInTab = function(aMessage) {
  var browser = aMessage.target;
  var tabBrowser = browser.getTabBrowser();
  var scriptTab = tabBrowser.getTabForBrowser(browser);
  var scriptTabIsCurrentTab = scriptTab == tabBrowser.mCurrentTab;
  // Work around a race condition in Firefox code with E10S disabled.
  // See #2107 and #2234
  // Todo: Remove timeout when http://bugzil.la/1200334 is resolved.
  AWE_util.timeout(function () {
    var newTab = tabBrowser.addTab(
        aMessage.data.url,
        {
            'ownerTab': scriptTab,
            'relatedToCurrent': scriptTabIsCurrentTab,
        });

    var getBool = Services.prefs.getBoolPref;

    var prefBg = (aMessage.data.inBackground === null)
        ? getBool("browser.tabs.loadInBackground")
        : aMessage.data.inBackground;
    if (scriptTabIsCurrentTab && !prefBg) tabBrowser.selectedTab = newTab;

    var prefRel = (aMessage.data.afterCurrent === null)
        ? getBool("browser.tabs.insertRelatedAfterCurrent")
        : aMessage.data.afterCurrent;
    if (prefRel) {
      tabBrowser.moveTabTo(newTab, scriptTab._tPos + 1);
    } else {
      tabBrowser.moveTabTo(newTab, tabBrowser.tabs.length - 1);
    }
  }, 0);
};

/**
 * The browser XUL has unloaded. Destroy references/watchers/listeners.
 */
AWE_BrowserUI.chromeUnload = function() {
  AWE_prefRoot.unwatch("enabled", AWE_BrowserUI.refreshStatus);
};

/**
 * Called when the content area context menu is showing. We figure out whether
 * to show our context items.
 */
AWE_BrowserUI.contextMenuShowing = function(aEvent) {
  AWE_ContextMenuCommander.onPopupShowing(aEvent);
  AWE_BrowserUI.getUserScriptUrlUnderPointer(function(aUrl) {
    var contextItem = document.getElementById("acewebextension-view-userscript");
    var contextSep = document.getElementById("acewebextension-install-sep");
    contextItem.hidden = contextSep.hidden = !aUrl;
  });
};

AWE_BrowserUI.getUserScriptUrlUnderPointer = function(callback) {
  var culprit = gContextMenu.target || document.popupNode;
  if (!culprit) {
    callback(null);
    return;
  }

  var mm = gBrowser.selectedBrowser.messageManager;
  var messageHandler;
  messageHandler = function (aMessage) {
    mm.removeMessageListener("acewebextension:context-menu-end", messageHandler);

    var href = aMessage.data.href;
    var userScriptPattern = AWE_prefRoot.getValue('handleUserJs')
        ? /\.(acestream|user)\.js(\?|$)/i
        : /\.acestream\.js(\?|$)/i;
    if (href && href.match(userScriptPattern)) {
      callback(href);
    } else {
      callback(null);
    }
  };
  mm.addMessageListener("acewebextension:context-menu-end", messageHandler);

  mm.sendAsyncMessage(
      "acewebextension:context-menu-start", {}, {"culprit": culprit});
};

AWE_BrowserUI.refreshStatus = function() {
  var enabledEl = document.getElementById("awe_toggle_enabled");
  var checkedEl = document.getElementById("awe_toggle_checked");

  if (AWE_util.getEnabled()) {
    checkedEl.setAttribute('checked', true);
    enabledEl.removeAttribute('disabled');
  } else {
    checkedEl.setAttribute('checked', false);
    enabledEl.setAttribute('disabled', 'yes');
  }
};

// Not used directly, kept for GreaseFire.  See #1507.
AWE_BrowserUI.startInstallScript = function(aUri) {
  AWE_util.showInstallDialog(aUri.spec, gBrowser);
};

AWE_BrowserUI.viewContextItemClicked = function() {
  AWE_BrowserUI.getUserScriptUrlUnderPointer(function(aUrl) {
    if (!aUrl) return;

    var scope = {};
    Components.utils.import(
        "chrome://acewebextension-modules/content/remoteScript.js", scope);
    var rs = new scope.RemoteScript(aUrl);
    rs.downloadScript(function (aSuccess) {
      if (aSuccess) {
        rs.showSource(gBrowser);
      } else {
        alert(rs.errorMessage);
      }
    });
  });
};

AWE_BrowserUI.showToolbarButton = function() {
  // See #1652.  During transition, this might be set, but not readable yet;
  // transition happens in an async callback to get addon version.  If existing
  // version is "0.0" (the default), this hasn't happened yet, so try later.
  if ('0.0' == AWE_prefRoot.getValue("version")) {
      setTimeout(function() {
              AWE_BrowserUI.showToolbarButton();
      }, 50);
    return;
  }

  // Once, enforce that the toolbar button is present.  For discoverability.
  if (!AWE_prefRoot.getValue('haveInsertedToolbarbutton')) {
    AWE_prefRoot.setValue('haveInsertedToolbarbutton', true);

    var navbar = document.getElementById("nav-bar");
    var newset = navbar.currentSet + ",acewebextension-tbb";
    navbar.currentSet = newset;
    navbar.setAttribute("currentset", newset);
    document.persist("nav-bar", "currentset");
  }
};

AWE_BrowserUI.openOptions = function() {
  openDialog('chrome://acewebextension/content/options.xul', null, 'modal');
};

AWE_BrowserUI.checkDisabledScriptNavigation = function(aEvent, aSafeWin, aHref) {
  var userScriptPattern = AWE_prefRoot.getValue('handleUserJs')
    ? /\.(acestream|user)\.js$/
    : /\.acestream\.js$/;
  if (!aHref.match(userScriptPattern)) return;
  if (aSafeWin.document.contentType.match(/^text\/(x|ht)ml/)) return;

  // Handle enabled (i.e. show script source button) navigation by default.
  var msg = AWE_BrowserUI.bundle.GetStringFromName('greeting.msg');
  var buttons = [];

  if (!AWE_util.getEnabled()) {
    // Add options for disabled state.
    msg = AWE_BrowserUI.bundle.GetStringFromName('disabledWarning');
    buttons.push({
      'label': AWE_BrowserUI.bundle.GetStringFromName('disabledWarning.enable'),
      'accessKey': AWE_BrowserUI.bundle.GetStringFromName('disabledWarning.enable.accessKey'),
      'popup': null,
      'callback': function() { AWE_util.setEnabled(true); }
    });
    buttons.push({
      'label': AWE_BrowserUI.bundle.GetStringFromName('disabledWarning.enableAndInstall'),
      'accessKey': AWE_BrowserUI.bundle.GetStringFromName('disabledWarning.enableAndInstall.accessKey'),
      'popup': null,
      'callback': function() {
        AWE_util.setEnabled(true);
        AWE_util.showInstallDialog(aHref, gBrowser);
      }
    });
  }

  buttons.push({
    'label': AWE_BrowserUI.bundle.GetStringFromName('disabledWarning.install'),
    'accessKey': AWE_BrowserUI.bundle.GetStringFromName('disabledWarning.install.accessKey'),
    'popup': null,
    'callback': AWE_util.hitch(this, function() {
      AWE_util.showInstallDialog(aHref, gBrowser);
    })
  });

  var notificationBox = gBrowser.getNotificationBox();
  notificationBox.appendNotification(
    msg,
    "install-userscript",
    "chrome://acewebextension/skin/icon16.png",
    notificationBox.PRIORITY_WARNING_MEDIUM,
    buttons
  );
};

AWE_BrowserUI.checkNewsForUrl = function(aEvent, aSafeWin, aHref) {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
    var scheme = ioService.extractScheme(aHref);
    var isChromeWin = false;
    
    try {
        aSafeWin.QueryInterface(Ci.nsIDOMChromeWindow);
        isChromeWin = true;
    }
    catch(e) {
    }
    
    if(isChromeWin) {
        return;
    }
    if(!aSafeWin.document.contentType.match(/^text\/(x|ht)ml/)) {
        return;
    }
    if(scheme != "http" && scheme != "https") {
        return;
    }
    
    var news = AWE_news.getNewsForUrl(aHref);
    var notificationBox = gBrowser.getNotificationBox();
    for(var i = 0; i < news.length; i++) {
        var buttons = [],
            newsId = news[i].id;
        
        if(news[i].btnUrl) {
            var url = news[i].btnUrl;
            buttons.push({
                    'label': news[i].btnTitle || "Show more",
                    'popup': null,
                    'callback': function() {
                        AWE_news.markAsRead(newsId);
                        gBrowser.selectedTab = gBrowser.addTab(url);
                    }
            });
        }
        
        buttons.push({
                'label': AWE_BrowserUI.bundle.GetStringFromName('news.markAsRead'),
                'popup': null,
                'callback': function() {
                    AWE_news.markAsRead(newsId);
                }
        });
        
        notificationBox.appendNotification(
            news[i].text,
            "awe-news",
            "chrome://acewebextension/skin/icon16.png",
            notificationBox.PRIORITY_WARNING_MEDIUM,
            buttons
            );
    }
};

AWE_BrowserUI.setVersion = function(aSafeWin, aVersion) {
    var el = aSafeWin.document.getElementById("x-acestream-awe-version");
    if(el) {
        console.log("setVersion: version=" + aVersion);
        el.setAttribute("data-vendor", "mozilla");
        el.setAttribute("data-version", aVersion);
    }
};

AWE_BrowserUI.setInstalledScripts = function(aSafeWin) {
    // expose installed scripts to a limited set of domains
    var el = aSafeWin.document.getElementById("x-acestream-awe-installed-scripts");
    if(el) {
        var allowedDomains = [
            "acestream.org",
            "acestream.net",
            "acestream.me"
        ],
        host = aSafeWin.location.host.split(".").slice(-2).join("."),
        allow = (allowedDomains.indexOf(host) != -1);
        
        var installedScripts = {};
        console.log("setInstalledScripts: host=" + host + " allow=" + allow + " installed=" + JSON.stringify(installedScripts));
        
        if(allow) {
            AWE_util.getService().config.getMatchingScripts(
                function(script) {
                    installedScripts[script.id] = true;
                });
            el.setAttribute("data-scripts", JSON.stringify(installedScripts));
        }
    }
};

AWE_BrowserUI.init();


/**
 * Handle clicking one of the items in the popup. Left-click toggles the enabled
 * state, right-click opens in an editor.
 */
function AWE_popupClicked(aEvent) {
  var script = aEvent.target.script;
  if (!script) return;

  if ('command' == aEvent.type) {
    // left-click: toggle enabled state
    script.enabled =! script.enabled;
  } else if ('click' == aEvent.type && aEvent.button == 2) {
    // right-click: open in editor
    AWE_util.openInEditor(script);
  }

  closeMenus(aEvent.target);
}

/**
 * When a menu pops up, fill its contents with the list of scripts.
 */
function AWE_showPopup(aEvent) {
  // Make sure this event was triggered by opening the actual monkey menu,
  // not one of its submenus.
  if (aEvent.currentTarget != aEvent.target) return;

  var mm = getBrowser().mCurrentBrowser.frameLoader.messageManager;

  var callback = null;
  callback = function(message) {
    mm.removeMessageListener("acewebextension:frame-urls", callback);

    var urls = message.data.urls;
    AWE_asyncShowPopup(aEvent, urls);
  };

  mm.addMessageListener("acewebextension:frame-urls", callback);
  mm.sendAsyncMessage("acewebextension:frame-urls", {});
}

function AWE_asyncShowPopup(aEvent, urls) {
  function uniq(a) {
    var seen = {}, list = [], item;
    for (var i = 0; i < a.length; i++) {
      item = a[i];
      if (!seen.hasOwnProperty(item))
        seen[item] = list.push(item);
    }
    return list;
  }

  function scriptsMatching(urls) {
    function testMatchURLs(script) {
      function testMatchURL(url) {
        return script.matchesURL(url);
      }
      return urls.some(testMatchURL);
    }
    return AWE_util.getService().config.getMatchingScripts(testMatchURLs);
  }

  function appendScriptAfter(script, point) {
    if (script.needsUninstall) return;
    var mi = document.createElement("menuitem");
    mi.setAttribute("label", script.localized.name);
    mi.script = script;
    mi.setAttribute("type", "checkbox");
    mi.setAttribute("checked", script.enabled.toString());
    point.parentNode.insertBefore(mi, point.nextSibling);
    return mi;
  }

  var popup = aEvent.target;
  var scriptsFramedEl = popup.getElementsByClassName("scripts-framed-point")[0];
  var scriptsTopEl = popup.getElementsByClassName("scripts-top-point")[0];
  var scriptsSepEl = popup.getElementsByClassName("scripts-sep")[0];
  var noScriptsEl = popup.getElementsByClassName("no-scripts")[0];

  // Remove existing menu items, between separators.
  function removeMenuitemsAfter(el) {
    while (true) {
      var sibling = el.nextSibling;
      if (!sibling || 'menuseparator' == sibling.tagName) break;
      sibling.parentNode.removeChild(sibling);
    }
  }
  removeMenuitemsAfter(scriptsFramedEl);
  removeMenuitemsAfter(scriptsTopEl);

  var runsOnTop = scriptsMatching( [urls.shift()] ); // first url = top window
  var runsFramed = scriptsMatching( urls ); // remainder are all its subframes

  // drop all runsFramed scripts already present in runsOnTop
  for (var i = 0; i < runsOnTop.length; i++) {
    var j = 0, item = runsOnTop[i];
    while (j < runsFramed.length) {
      if (item === runsFramed[j]) {
        runsFramed.splice(j, 1);
      } else {
        j++;
      }
    }
  }

  scriptsSepEl.collapsed = !(runsOnTop.length && runsFramed.length);
  noScriptsEl.collapsed = !!(runsOnTop.length || runsFramed.length);

  var point;
  if (runsFramed.length) {
    point = scriptsFramedEl;
    runsFramed.forEach(
        function(script) { point = appendScriptAfter(script, point); });
  }
  point = scriptsTopEl;
  runsOnTop.forEach(
      function(script) { point = appendScriptAfter(script, point); });

  // Propagate to commands sub-menu.
  AWE_MenuCommander.onPopupShowing(aEvent);
}

/**
 * Clean up the menu after it hides to prevent memory leaks
 */
function AWE_hidePopup(aEvent) {
  // Only handle the actual monkey menu event.
  if (aEvent.currentTarget != aEvent.target) return;
  // Propagate to commands sub-menu.
  AWE_MenuCommander.onPopupHiding();
}

// Short-term workaround for #1406: Tab Mix Plus breaks opening links in
// new tabs because it depends on this function, and incorrectly checks for
// existance of AWE_BrowserUI instead of it.
function AWE_getEnabled() {
  return AWE_util.getEnabled();
}

function windowIsTop(aContentWin) {
  try {
    aContentWin.QueryInterface(Ci.nsIDOMWindow);
    if (aContentWin.frameElement) return false;
  } catch (e) {
    var url = 'unknown';
    try {
      url = aContentWin.location.href;
    } catch (e) { }
    // Ignore non-DOM-windows.
    dump('Could not QI window to nsIDOMWindow at\n' + url + ' ?!\n');
  }
  return true;
};
