// The frame script for Electrolysis (e10s) compatible injection.
//   See: https://developer.mozilla.org/en-US/Firefox/Multiprocess_Firefox

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;
var AWE = {};

Cu.import('resource://gre/modules/XPCOMUtils.jsm');

Cu.import('chrome://acewebextension-modules/content/installPolicy.js', AWE);
Cu.import('chrome://acewebextension-modules/content/ipcscript.js', AWE);
Cu.import('chrome://acewebextension-modules/content/menucommand.js', AWE);
Cu.import('chrome://acewebextension-modules/content/contextmenucommand.js', AWE);
Cu.import('chrome://acewebextension-modules/content/sandbox.js', AWE);
Cu.import('chrome://acewebextension-modules/content/scriptProtocol.js', AWE);
Cu.import('chrome://acewebextension-modules/content/documentObserver.js', AWE);
Cu.import('chrome://acewebextension-modules/content/util.js');

// register with process script. no need to import all the vars into the local scope
Cu.import('chrome://acewebextension-modules/content/processScript.js', {}).addFrame(this);

// \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ //

var gScope = this;
var gStripUserPassRegexp = new RegExp('(://)([^:/]+)(:[^@/]+)?@');

// \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ //

function contentObserver(win) {
  if (!AWE_util.getEnabled()) return;

  var doc = win.document;
  var url = doc.documentURI;
  if (!AWE_util.isGreasemonkeyable(url)) return;

  // Listen for whichever kind of load event arrives first.
  win.addEventListener('DOMContentLoaded', contentLoad, true);
  win.addEventListener('load', contentLoad, true);

  runScripts('document-start', win);
};

// \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ //

function blankLoad(aEvent) {
  var contentWin = aEvent.target.defaultView;
  if (contentWin && contentWin.location.href.match(/^about:(blank|reader)/)) {
    // #1696: document-element-inserted doesn't see about:blank
    runScripts('document-start', contentWin);
    runScripts('document-end', contentWin);
  }
}


function contentLoad(aEvent) {
  var contentWin = aEvent.target.defaultView;

  // Now that we've seen any first load event, stop listening for any more.
  contentWin.removeEventListener('DOMContentLoaded', contentLoad, true);
  contentWin.removeEventListener('load', contentLoad, true);

  runScripts('document-end', contentWin);
}


function createScriptFromObject(aObject) {
  var script = Object.create(AWE.IPCScript.prototype);
  // TODO: better way for this? Object.create needs property descriptors.
  for (var key in aObject) {
    script[key] = aObject[key];
  }
  return script;
};


function injectDelayedScript(aMessage) {
  var windowId = aMessage.data.windowId;
  var windowMediator = Components
      .classes['@mozilla.org/appshell/window-mediator;1']
      .getService(Components.interfaces.nsIWindowMediator);
  var win = windowMediator.getOuterWindowWithId(windowId);

  if (!win) {
    dump('Couldn\'t find window with (outer?) ID ' + windowId + '!\n');
  } else {
    var script = createScriptFromObject(aMessage.data.script);
    injectScripts([script], win);
  }
};


function injectScripts(aScripts, aContentWin) {
  try {
    aContentWin.QueryInterface(Ci.nsIDOMChromeWindow);
    // Never ever inject scripts into a chrome context window.
    return;
  } catch(e) {
    // Ignore, it's good if we can't QI to a chrome window.
  }

  var url = urlForWin(aContentWin);
  var winIsTop = windowIsTop(aContentWin);

  for (var i = 0, script = null; script = aScripts[i]; i++) {
    if (script.noframes && !winIsTop) continue;
    var sandbox = AWE.createSandbox(script, aContentWin, url, gScope);
    AWE.runScriptInSandbox(script, sandbox);
  }
}


function loadFailedScript(aMessage) {
  var url = aMessage.data.url;
  var loadFlags = Ci.nsIWebNavigation.LOAD_FLAGS_NONE;
  var referer = aMessage.data.referer
      && AWE_util.uriFromUrl(aMessage.data.referer);
  var postData = null;
  var headers = null;

  var webNav = docShell.QueryInterface(Ci.nsIWebNavigation);

  AWE.ignoreNextScript();
  webNav.loadURI(url, loadFlags, referer, postData, headers);
}


function contextMenuStart(aMessage) {
  var culprit = aMessage.objects.culprit;

  while (culprit && culprit.tagName && culprit.tagName.toLowerCase() != "a") {
    culprit = culprit.parentNode;
  }

  aMessage.target.sendAsyncMessage(
      "acewebextension:context-menu-end", {"href": culprit.href});
}


function newScriptLoadStart(aMessage) {
  aMessage.target.sendAsyncMessage(
      "acewebextension:newscript-load-end", {"href": content.location.href});
}


function runScripts(aRunWhen, aContentWin) {
  var url = urlForWin(aContentWin);
  if (!AWE_util.isGreasemonkeyable(url)) return;

  var response = sendSyncMessage(
    'acewebextension:scripts-for-url', {
      'url': url,
      'when': aRunWhen,
      'windowId': AWE_util.windowId(aContentWin, 'outer'),
    });
  if (!response || !response[0]) return;

  var scripts = response[0].map(createScriptFromObject);
  injectScripts(scripts, aContentWin);
}


function urlForWin(aContentWin) {
  // See #1970
  // When content does (e.g.) history.replacestate() in an inline script,
  // the location.href changes between document-start and document-end time.
  // But the content can call replacestate() much later, too.  The only way to
  // be consistent is to ignore it.  Luckily, the  document.documentURI does
  // _not_ change, so always use it when deciding whether to run scripts.
  var url = aContentWin.document.documentURI;
  // But ( #1631 ) ignore user/pass in the URL.
  return url.replace(gStripUserPassRegexp, '$1');
}


function windowIsTop(aContentWin) {
  try {
    aContentWin.QueryInterface(Ci.nsIDOMWindow);
    if (aContentWin.frameElement) return false;
  } catch (e) {
    var url = 'unknown';
    try {
      url = aContentWin.location.href;
    } catch (e) { }
    // Ignore non-DOM-windows.
    dump('Could not QI window to nsIDOMWindow at\n' + url + ' ?!\n');
  }
  return true;
};


function windowCreated() {
  AWE.onNewDocument(content, contentObserver);
}

// \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ // \\ //

addEventListener('DOMContentLoaded', blankLoad);
addEventListener('DOMWindowCreated', windowCreated);

if (content) windowCreated();

addMessageListener('acewebextension:inject-delayed-script', injectDelayedScript);
addMessageListener('acewebextension:load-failed-script', loadFailedScript);
addMessageListener('acewebextension:menu-command-list', function(aMessage) {
  AWE.MenuCommandListRequest(content, aMessage);
});
addMessageListener('acewebextension:menu-command-run', function(aMessage) {
  AWE.MenuCommandRun(content, aMessage);
});
addMessageListener('acewebextension:context-menu-command-list', function(aMessage) {
  AWE.ContextMenuCommandListRequest(content, aMessage);
});
addMessageListener('acewebextension:context-menu-command-run', function(aMessage) {
  AWE.ContextMenuCommandRun(content, aMessage);
});
addMessageListener("acewebextension:context-menu-start", contextMenuStart);
addMessageListener("acewebextension:newscript-load-start", newScriptLoadStart);


AWE.initInstallPolicy();
AWE.initScriptProtocol();
