var EXPORTED_SYMBOLS = [
    'ContextMenuCommandListRequest',
    'ContextMenuCommandRespond',
    'ContextMenuCommandRun',
    'ContextMenuCommandSandbox',
    ];


var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;

var consoleService = Cc["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
Cu.import('chrome://acewebextension-modules/content/menucommand.js');

// Frame scope: Pass "list menu commands" message into sandbox as event.
function ContextMenuCommandListRequest(aContent, aMessage) {
  var e = new aContent.CustomEvent(
      'acewebextension-context-menu-command-list-' + MenuCommandEventNameSuffix,
      {'detail': JSON.stringify(aMessage.data)});
  aContent.dispatchEvent(e);
}


// Callback from script scope, pass "list menu commands" response up to
// parent process as a message.
function ContextMenuCommandRespond(aCookie, aData) {
  var cpmm = Cc["@mozilla.org/childprocessmessagemanager;1"]
      .getService(Ci.nsIMessageSender);
      
  cpmm.sendAsyncMessage(
      'acewebextension:context-menu-command-response',
      {'commands': aData, 'cookie': aCookie});
}


// Frame scope: Respond to the "run this menu command" message coming
// from the parent, pass it into the sandbox.
function ContextMenuCommandRun(aContent, aMessage) {
  var e = new aContent.CustomEvent(
      'acewebextension-context-menu-command-run-' + MenuCommandEventNameSuffix,
      {'detail': JSON.stringify(aMessage.data)});
  aContent.dispatchEvent(e);
}


// This function is injected into the sandbox, in a private scope wrapper, BY
// SOURCE.  Data and sensitive references are wrapped up inside its closure.
function ContextMenuCommandSandbox(
    aScriptUuid, aScriptName, aCommandResponder, aInvalidAccesskeyErrorStr,
    aMenuCommandEventNameSuffix) {
  // 1) Internally to this function's private scope, maintain a set of
  // registered menu commands.
  var commands = {};
  var commandFuncs = {};
  var commandFilters = {};
  var commandCookie = 0;
  
  function getFilteredCommands(url, callback) {
      var cookie, command, filteredCommands = {};
      for(cookie in commands) {
          //TODO: support multiple commands (need to wait for all filter callbacks)
          var command = commands[cookie];
          if(!commandFilters[cookie]) {
              filteredCommands[cookie] = command;
              callback(filteredCommands);
          }
          else {
              commandFilters[cookie].call(null, url, function(enabled) {
                      if(enabled) {
                          filteredCommands[cookie] = command;
                          callback(filteredCommands);
                      }
              });
          }
      }
  }
  
  // 2) Respond to requests to list those registered commands.
  addEventListener(
      'acewebextension-context-menu-command-list-' + aMenuCommandEventNameSuffix,
      function(e) {
        console.log("contextmenu:acewebextension-context-menu-command-list: detail=" + e.detail);
        e.stopPropagation();
        var detail = JSON.parse(e.detail);
        getFilteredCommands(detail.url, function(filteredCommands) {
                if(filteredCommands) {
                    aCommandResponder(detail.cookie, filteredCommands);
                }
        });
      }, true);
  // 3) Respond to requests to run those registered commands.
  addEventListener(
      'acewebextension-context-menu-command-run-' + aMenuCommandEventNameSuffix,
      function(e) {
        console.log("contextmenu:acewebextension-context-menu-command-run: detail=" + e.detail); 
        e.stopPropagation();
        var detail = JSON.parse(e.detail);
        if (aScriptUuid != detail.scriptUuid) return;
        // This event is for this script; stop propagating to other scripts.
        e.stopImmediatePropagation();
        var commandFunc = commandFuncs[detail.cookie];
        if (!commandFunc) {
          throw new Error('Could not run requested menu command!');
        } else {
          commandFunc.call(null, detail.url);
        }
      }, true);
  // 4) Export the "register a command" API function to the sandbox scope.
  this.AWE_registerContextMenuCommand = function(
      commandName, commandFunc, accessKey, commandFilter) {

    if (accessKey
        && (("string" != typeof accessKey) || (accessKey.length != 1))
    ) {
      throw new Error(aInvalidAccesskeyErrorStr.replace('%1', commandName));
    }

    var command = {
      cookie: ++commandCookie,
      name: commandName,
      scriptName: aScriptName,
      scriptUuid: aScriptUuid,
      accessKey: accessKey,
    };
    commands[command.cookie] = command;
    commandFuncs[command.cookie] = commandFunc;
    if(typeof commandFilter === "function") {
      commandFilters[command.cookie] = commandFilter;
    }
  };
}
