var EXPORTED_SYMBOLS = ['AWE_engineAPI'];

Components.utils.import('chrome://acewebextension-modules/content/engineStatus.js');
Components.utils.import('chrome://acewebextension-modules/content/util.js');

var consoleService = Components
    .classes["@mozilla.org/consoleservice;1"]
    .getService(Components.interfaces.nsIConsoleService);
    
var AWE_engineAPI = (function() {
        function _makeParams(params) {
            var name, value, result = [];
            for(name in params) {
                value = params[name];
                result.push(name + "=" + encodeURIComponent(value));
            }
            return result.join("&");
        }
        
        function _sendRequestWithEngineCheck(details)
        {
            AWE_engineStatus.get(function(engineStatus) {
                    consoleService.logStringMessage("_sendRequestWithEngineCheck: engineStatus=" + JSON.stringify(engineStatus));
                    if(engineStatus.running) {
                        _sendRequest(details);
                    }
                    else {
                        if(typeof details.onerror === "function") {
                            details.onerror.call(null, "engine is not running");
                        }
                    }
            },
            // async
            true);
        }
        
        function _sendRequest(details)
        {
            function _success(data) {
                if(typeof details.onsuccess === "function") {
                    details.onsuccess.call(null, data);
                }
            }
            
            function _failed(errmsg) {
                consoleService.logStringMessage("engineapi:request failed: " + errmsg);
                if(typeof details.onerror === "function") {
                    details.onerror.call(null, errmsg);
                }
            }
            
            try {
                if(!details.method) {
                    throw "missing method";
                }
                if(!details.params) {
                    details.params = {};
                }
                if(!details.api) {
                    details.api = "server";
                }
                details.params.method = details.method;
                
                var baseUrl;
                if(details.api == "server") {
                    baseUrl = "http://127.0.0.1:6878/server/api"
                }
                else if(details.api == "service") {
                    baseUrl = "http://127.0.0.1:6878/webui/api/service"
                }
                else {
                    throw "unknown api: " + details.api;
                }
                
                var url = baseUrl + "?" + _makeParams(details.params);
                var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
                    .createInstance(Components.interfaces.nsIXMLHttpRequest);
                req.overrideMimeType('application/json');
                req.timeout = 10000;  // milliseconds
                req.open("GET", url, true);
                
                req.onload = function() {
                    if(req.status == 200) {
                        try {
                            var data = JSON.parse(req.responseText);
                            if(data.error) {
                                _failed(data.error);
                            }
                            else if(!data.result) {
                                _failed("malformed response from engine");
                            }
                            else {
                                _success(data.result);
                            }
                        }
                        catch(e) {
                            _failed(e);
                        }
                    }
                    else {
                        _failed();
                    }
                }
                req.onerror = function() {
                    _failed();
                }
                req.ontimeout = function() {
                    _failed();
                }
                
                req.send(null);
            }
            catch(e) {
                AWE_util.logError(e);
            }
        }
        
        return {
            getAvailablePlayers: function(sandbox, details, callback) {
                if(typeof callback !== "function") {
                    return;
                }
                
                var params = {};
                if(details.content_id) {
                    params['content_id'] = details.content_id;
                }
                else if(details.transport_file_url) {
                    params['url'] = details.transport_file_url;
                }
                else {
                    callback.call(null);
                }
                
                _sendRequest({
                        method: "get_available_players",
                        params: params,
                        onsuccess: function(data) {
                            var ret = Components.utils.cloneInto(data, sandbox);
                            callback.call(null, ret);
                        },
                        onerror: function(errmsg) {
                            callback.call(null);
                        }
                });
            },
            openInPlayer: function(sandbox, details, playerId, callback) {
                var params = {};
                if(details.content_id) {
                    params['content_id'] = details.content_id;
                }
                else if(details.transport_file_url) {
                    params['url'] = details.transport_file_url;
                }
                else {
                    callback.call(null);
                }
                if(playerId) {
                    params['player_id'] = playerId;
                }
                
                _sendRequest({
                        method: "open_in_player",
                        params: params,
                        onsuccess: function(data) {
                            if(typeof callback === "function") {
                                var ret = Components.utils.cloneInto(data, sandbox);
                                callback.call(null, ret);
                            }
                        },
                        onerror: function(errmsg) {
                            if(typeof callback === "function") {
                                callback.call(null);
                            }
                        }
                });
            },
            getDeviceId: function(sandbox, callback) {
                _sendRequest({
                        api: "service",
                        method: "get_public_user_key",
                        onsuccess: function(data) {
                            if(typeof callback === "function") {
                                var ret = Components.utils.cloneInto(data, sandbox);
                                callback.call(null, ret);
                            }
                        },
                        onerror: function(errmsg) {
                            if(typeof callback === "function") {
                                callback.call(null);
                            }
                        }
                });
            }
        };
})();


