var EXPORTED_SYMBOLS = ['AWE_engineStatus'];

Components.utils.import("resource://gre/modules/FileUtils.jsm");
Components.utils.import('chrome://acewebextension-modules/content/util.js');

var consoleService = Components
    .classes["@mozilla.org/consoleservice;1"]
    .getService(Components.interfaces.nsIConsoleService);
    
var timer = Components.classes["@mozilla.org/timer;1"]
    .createInstance(Components.interfaces.nsITimer);
    
var AWE_engineStatus = (function() {
        var initDone = false,
            triedToRunEngineAt_ = 0,
            engineRunning_ = false,
            engineVersionCode_ = 0;
            
        function gotResponse(req, status, callback, runEngine, retries)
        {
            var retries = retries || 0,
                running = true,
                versionCode = 0;
                
            consoleService.logStringMessage("enginestatus: got response: status=" + status + " http_status=" + req.status + " data=" + req.responseText + " run=" + runEngine + " retries=" + retries);
            
            if(status === "ok") {
                if(req.status === 200) {
                    try {
                        var response = JSON.parse(req.responseText);
                        if(response.result) {
                            versionCode = parseInt(response.result.code);
                            if(isNaN(versionCode)) {
                                versionCode = 0;
                            }
                        }
                    }
                    catch(e) {
                        AWE_util.logError(e);
                    }
                }
                else if(req.status === 0) {
                    running = false;
                }
            }
            else {
                running = false;
            }
            
            if(!running && runEngine) {
                try {
                    // run engine if it's not running
                    consoleService.logStringMessage("run engine");
                    triedToRunEngineAt_ = new Date().getTime();
                    var file = FileUtils.getFile("AppData", ["ACEStream", "engine", "ace_engine.exe"]);
                    
                    var process = Components.classes["@mozilla.org/process/util;1"]
                        .createInstance(Components.interfaces.nsIProcess);
                    process.init(file);
                    process.run(false, [], 0);
                    
                    // give engine some time to start and check 3 times with interval 3 seconds
                    retries = 3;
                }
                catch(e) {
                    consoleService.logStringMessage("failed to run engine: " + e);
                }
            }
            
            engineRunning_ = running;
            engineVersionCode_ = versionCode;
            
            if(!running && retries > 0) {
                // Engine is not running and we still have retries. Schedule
                // nex check and don't call callback.
                AWE_util.timeout(function() {
                        check(callback, false, retries-1);
                }, 3000);
                return;
            }
            
            if(typeof callback === "function") {
                callback.call(null, {
                        running: engineRunning_,
                        version: engineVersionCode_
                });
            }
        }
        
        function check(callback, runEngine, retries)
        {
            try {
                var url = "http://127.0.0.1:6878/webui/api/service?method=get_version";
                var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
                    .createInstance(Components.interfaces.nsIXMLHttpRequest);
                req.overrideMimeType('application/json');
                req.timeout = 10000;  // milliseconds
                req.open("GET", url, true);
                
                req.onload = AWE_util.hitch(null, gotResponse, req, "ok", callback, runEngine, retries);
                req.onerror = AWE_util.hitch(null, gotResponse, req, "error", callback, runEngine, retries);
                req.ontimeout = AWE_util.hitch(null, gotResponse, req, "timeout", callback, runEngine, retries);
                
                req.send(null);
            }
            catch(e) {
                AWE_util.logError(e);
            }
        }
        
        return {
            init: function() {
                if(initDone) {
                    return;
                }
                
                initDone = true;
                check();
                
                var observer = {
                    'observe': function() {
                        check();
                    }
                };
                // check engine every minute
                timer.init(observer, 60000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
            },
            get: function(sandbox, callback, async) {
                var runEngine = false;
                if(typeof callback !== "function") {
                    consoleService.logStringMessage("engineStatus:get: missing callback");
                    return;
                }
                
                if(!engineRunning_) {
                    // try to run engine not often then once an hour
                    var now = new Date().getTime(),
                        age = now - triedToRunEngineAt_;
                    if(age > 3600000) {
                        runEngine = true;
                        async = true;
                    }
                    consoleService.logStringMessage("engineStatus:get: runEngine=" + runEngine + " age=" + age/1000);
                }
                
                if(async) {
                    // get async engine status now
                    check(function(response) {
                            var ret = Components.utils.cloneInto(response, sandbox);
                            callback(ret);
                    }, runEngine);
                }
                else {
                    // get last engine status
                    var status = {
                        running: engineRunning_,
                        version: engineVersionCode_
                    };
                    var ret = Components.utils.cloneInto(status, sandbox);
                    callback(ret);
                }
            }
        };
})();


