var EXPORTED_SYMBOLS = ['AWE_news'];

Components.utils.import('chrome://acewebextension-modules/content/util.js');

var consoleService = Components
    .classes["@mozilla.org/consoleservice;1"]
    .getService(Components.interfaces.nsIConsoleService);
    
var timer = Components.classes["@mozilla.org/timer;1"]
    .createInstance(Components.interfaces.nsITimer);
    
function getLocale() {
    var locale;
    try {
        locale = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
            .getService(Components.interfaces.nsIXULChromeRegistry)
            .getSelectedLocale('global'); // e.g "en-US"
    }
    catch(e) {
        locale = "";
    }
    return locale;
}

var AWE_news = (function() {
        var debug_ = false,
            initDone_ = false,
            configFile_,
            news_ = {};
            
        configFile_ = AWE_util.scriptDir();
        configFile_.append("news.json")
        
        function loadConfig() {
            if(configFile_.exists()) {
                news_ = JSON.parse(AWE_util.getContents(configFile_));
            }
        }
        
        function saveConfig() {
            AWE_util.writeToFile(JSON.stringify(news_), configFile_);
        }

        function check()
        {
            try {
                var url = "http://awe-api.acestream.me/news/get?locale=" + getLocale() + "&_=" + Math.random();
                var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
                    .createInstance(Components.interfaces.nsIXMLHttpRequest);
                req.timeout = 10000;  // 10 seconds
                req.open("GET", url, true);
                
                req.onload = function() {
                    if(req.status === 200) {
                        try {
                            var updated = false;
                            remote = JSON.parse(req.responseText);
                            
                            for(var id in remote) {
                                if(!news_[id]) {
                                    news_[id] = remote[id];
                                    news_[id].read = false;
                                    updated = true;
                                }
                            }
                            
                            for(var id in news_) {
                                if(!remote[id]) {
                                    delete news_[id];
                                    updated = true;
                                }
                            }
                            
                            if(updated) {
                                saveConfig();
                            }
                        }
                        catch(e) {
                            AWE_util.logError(e);
                        }
                    }
                };
                
                req.send(null);
            }
            catch(e) {
                AWE_util.logError(e);
            }
        }
        
        return {
            init: function() {
                if(initDone_) {
                    return;
                }
                
                initDone_ = true;
                loadConfig();
                check();
                
                var observer = {
                    'observe': function() {
                        check();
                    }
                };
                // check engine every four hours
                timer.init(observer, 14400000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
            },
            getNewsForUrl: function(url) {
                var i,
                    newsModified = false,
                    result = [];
                    
                for(var id in news_) {
                    if(!news_[id].read) {
                        var gotMatch = false;
                        if(news_[id].includes && news_[id].includes.length) {
                            if(debug_) {
                                consoleService.logStringMessage("getNewsForUrl: match against includes: url=" + url);
                            }
                            for(i=0; i < news_[id].includes.length; i++) {
                                var re = new RegExp(news_[id].includes[i]);
                                if(re.test(url)) {
                                    if(debug_) {
                                        consoleService.logStringMessage("getNewsForUrl: got includes match: re=" + news_[id].includes[i] + " url=" + url);
                                    }
                                    gotMatch = true;
                                    break;
                                }
                            }
                        }
                        else if(news_[id].excludes && news_[id].excludes.length) {
                            gotMatch = true;
                            if(debug_) {
                                consoleService.logStringMessage("getNewsForUrl: match against excludes: url=" + url);
                            }
                            for(i=0; i < news_[id].excludes.length; i++) {
                                var re = new RegExp(news_[id].excludes[i]);
                                if(re.test(url)) {
                                    if(debug_) {
                                        consoleService.logStringMessage("getNewsForUrl: got excludes match: re=" + news_[id].excludes[i] + " url=" + url);
                                    }
                                    gotMatch = false;
                                    break;
                                }
                            }
                        }
                        
                        if(gotMatch) {
                            var notifyUser = true;
                            
                            if(news_[id].excludeScripts && news_[id].excludeScripts.length) {
                                // check all installed scripts
                                var installedScripts = {};
                                
                                var scripts = AWE_util.getService().config.getMatchingScripts(
                                    function(script) {
                                        installedScripts[script.id] = 1;
                                    });
                                
                                if(debug_) {
                                    consoleService.logStringMessage("getNewsForUrl: installedScripts=" + JSON.stringify(installedScripts));
                                }
                                for(i = 0; i < news_[id].excludeScripts.length; i++) {
                                    if(installedScripts[news_[id].excludeScripts[i]] === 1) {
                                        if(debug_) {
                                            consoleService.logStringMessage("getNewsForUrl: skip user notify, got installed script: " + news_[id].excludeScripts[i]);
                                        }
                                        notifyUser = false;
                                        break;
                                    }
                                }
                            }
                        
                            if(notifyUser) {
                                result.push({
                                        id: id,
                                        title: news_[id].title,
                                        text: news_[id].text,
                                        btnUrl: news_[id].btnUrl,
                                        btnTitle: news_[id].btnTitle
                                });
                            }
                            else {
                                // we got news about the script which is already
                                // installed so mark is as read to never show again
                                news_[id].read = true;
                                newsModified = true;
                            }
                        }
                    }
                }
                
                if(newsModified) {
                    saveConfig();
                }
                
                return result;
            },
            markAsRead: function(id) {
                if(news_[id] && !news_[id].read) {
                    news_[id].read = true;
                    saveConfig();
                }
            }
        };
})();


